/*_______________________________________________________________

TC-04.C

Purpose:  Draws a solid 3D sphere using the plane equation
method of hidden surface removal.

Compatibility:  Supports VGA and EGA graphics adapters and
monitors.  Uses the 640x480 16-color mode if a VGA is present,
the 640x350 16-color mode if an EGA and enhanced monitor are
present, the 640x200 16-color mode if an EGA and standard
monitor are present.

Remarks:  Requires Borland Turbo C 1.5 or newer.

Copyright 1989 Lee Adams and TAB BOOKS Inc.

_________________________________________________________________


I N C L U D E    F I L E S                                       */

#include <process.h>               /* supports the exit function */
#include <bios.h>                 /* supports keyboard functions */
#include <stdio.h>               /* supports the printf function */
#include <graphics.h>         /* supports the graphics functions */
#include <math.h>          /* supports sine and cosine functions */

/*_______________________________________________________________


D E C L A R A T I O N S                                          */

float x=0.0,y=0.0,z=0.0;                    /* world coordinates */
float x1=0.0,x2=0.0,x3=0.0,x4=0.0;           /* polygon vertices */
float y01=0.0,y2=0.0,y3=0.0,y4=0.0;          /* polygon vertices */
float z1=0.0,z2=0.0,z3=0.0,z4=0.0;           /* polygon vertices */
float sx1=0.0,sx2=0.0,sx3=0.0,sx4=0.0,sx5=0.0; /* display coords */
float sy1=0.0,sy2=0.0,sy3=0.0,sy4=0.0,sy5=0.0; /* display coords */
float sx=0.0,sy=0.0;        /* output of 3D perspective formulas */
float xa=0.0,ya=0.0,za=0.0;   /* temporary values in 3D formulas */
float d=1200.0;                    /* angular perspective factor */
double r1=6.19592;                       /* yaw angle in radians */
double r2=.523590;                      /* roll angle in radians */
double r3=5.39778;                     /* pitch angle in radians */
double sr1=0.0,sr2=0.0,sr3=0.0;         /* sine rotation factors */
double cr1=0.0,cr2=0.0,cr3=0.0;       /* cosine rotation factors */
float mx=0.0,my=0.0,mz=-150.0;             /* viewpoint position */
int maxx=639,minx=0,maxy=199,miny=0;         /* scaling viewport */
float screen_x=639,screen_y=199;    /* dimensions of screen mode */
float rx=0.0,ry=0.0;   /* scaling values used in mapping routine */
int t=0,t1=0,t2=0;                              /* loop counters */
int p1=0;                                       /* array indexer */
int C0=0,C1=1,C2=2,C3=3,C4=4,C5=5,C6=6,C7=7,C8=8,C9=9,C10=10,
				C11=11,C12=12,C13=13,C14=14,C15=15,mode_flag=0;
int key_matte_clr=6;                /* exclusive key matte color */
int edge_clr=7;                  /* used to draw edges of models */
int solid_clr=0;        /* used to fill surfaces of solid models */
float x_res,y_res;  /* used for 2D mapping from 640x480 template */
float sp=0.0;     /* visibility factor in hidden surface routine */
float sp1=0.0,sp2=0.0,sp3=0.0;         /* temporary values of sp */
float xlarge=0.0,xsmall=0.0; /* used when finding polygon center */
float ylarge=0.0,ysmall=0.0;
float zlarge=0.0,zsmall=0.0;
float x6=0.0,x7=0.0,y6=0.0,y7=0.0,z6=0.0,z7=0.0;

int q=0,q1=0,q2=0;           /* sphere surface & vertex counters */
double r4=6.28319,r5=6.28319;     /* spherical coordinate angles */
float B11[36][3];           /* 36 sets of xyz view coords, upper */
float B12[36][3];           /* 36 sets of xyz view coords, lower */
float B21[36][2];         /* 36 sets sx,sy display coords, upper */
float B22[36][2];         /* 36 sets sx,sy display coords, lower */
double sr4=0.0,cr4=0.0,sr5=0.0,cr5=0.0;      /* rotation factors */

																	/* declare global subroutines */
void keyboard(void);void quit_pgm(void);void calc_3d(void);
void rotation(void);void window(void);void graphics_setup(void);
void coords(void);void draw_poly(void);void notice(int x,int y);
void visibility_test(void);void polycenter(void);

void sphere_coords(void);void draw_4poly(void);
void draw_3polynorth(void);void draw_3polysouth(void);
void upper_coords(void);void lower_coords(void);

/*_______________________________________________________________


M A I N    R O U T I N E                                         */

main(){
graphics_setup();                     /* establish graphics mode */
setviewport(0,0,maxx,maxy,1);      /* define clipping boundaries */
key_matte_clr=C6;                      /* define key matte color */
edge_clr=C7;solid_clr=C0;              /* define modeling colors */
rotation();            /* calculate the yaw, roll, pitch factors */

/*                  draw north polar region                      */
r5=0.0;r4=0.0;x=30;sphere_coords();calc_3d();window();
B11[0][0]=x;B11[0][1]=y;B11[0][2]=z;   /* north pole view coords */
B21[0][0]=sx;B21[0][1]=sy;          /* north pole display coords */
x=30;r5=.17453;r4=0.0;lower_coords(); /* store lower belt coords */
for (q1=0;q1<=35;q1++){                /* repeat for 36 polygons */
		q2=q1+1;if (q2>35) q2=0;             /* define array indexer */
		draw_3polynorth();               /* draw one 3-sided polygon */
		keyboard();
}                                      /* logical end of q1 loop */

/*             draw polygons along belts of sphere               */
r5=.17453;
for (t2=1;t2<=16;t2++){                   /* repeat for 16 belts */
		x=30;r4=0.0;upper_coords();  /* store vertices for upper row */
		x=30;r5=r5+.17453;r4=0.0;lower_coords();/* and for lower row */
		for (q1=0;q1<=35;q1++){         /* draw 36 polygons per belt */
		q2=q1+1;if (q2>35) q2=0;         /* define array indexer */
		draw_4poly();                /* draw one 4-sided polygon */
		keyboard();
		}                                  /* logical end of q1 loop */
}                                      /* logical end of t2 loop */

/*                  draw south polar region                      */
r5=3.14159;r4=0.0;x=30;sphere_coords();calc_3d();window();
B11[0][0]=x;B11[0][1]=y;B11[0][2]=z;   /* south pole view coords */
B21[0][0]=sx;B21[0][1]=sy;          /* south pole display coords */
x=30;r5=2.96706;r4=0.0;upper_coords();/* store upper belt coords */
for (q1=0;q1<=35;q1++){                /* repeat for 36 polygons */
		q2=q1+1;if (q2>35) q2=0;             /* define array indexer */
		draw_3polysouth();               /* draw one 3-sided polygon */
		keyboard();
}                                      /* logical end of q1 loop */

setcolor(C7);notice(0,0);
for (t1=1;t1!=2; ) keyboard();          /* press any key to stop */
quit_pgm();}                       /* end the program gracefully */

/*_______________________________________________________________


SUBROUTINE: CALCULATE AND STORE COORDS FOR BELT                  */

void upper_coords(void){   /* calculate vertices along upper row */
for (t=0;t<=35;t++){
		x=30;sphere_coords();calc_3d();window(); /* calculate coords */
		B11[t][0]=x;B11[t][1]=y;B11[t][2]=z;    /* store view coords */
		B21[t][0]=sx;B21[t][1]=sy;      /* store display coordinates */
		r4=r4+.17453;        /* increment yaw rotation by 10 degrees */
		keyboard();}
return;}

void lower_coords(void){   /* calculate vertices along lower row */
for (t=0;t<=35;t++){
		x=30;sphere_coords();calc_3d();window(); /* calculate coords */
		B12[t][0]=x;B12[t][1]=y;B12[t][2]=z;    /* store view coords */
		B22[t][0]=sx;B22[t][1]=sy;      /* store display coordinates */
		r4=r4+.17453;        /* increment yaw rotation by 10 degrees */
		keyboard();}
return;}

/*_______________________________________________________________


SUBROUTINE: CALCULATE WORLD COORDS FOR SPHERE
This subroutine uses sine and cosine to calculate the x,y,z
coordinates of various vertices on the surface of the 3D sphere.
The variables r4 and r5 refer to yaw and roll in the 3D
spherical coordinate system, respectively.                       */

void sphere_coords(void){sr4=sin(r4);cr4=cos(r4);sr5=sin(r5);
cr5=cos(r5);x1=sr5*x;y=cr5*x;x=cr4*x1;z=sr4*x1;return;}

/*_______________________________________________________________


SUBROUTINE: DRAW 4-sided POLYGONAL SURFACE ON SPHERE
This subroutine retrieves the pre-calculated view coordinates
from the appropriate arrays, tests for hidden surfaces, retrieves
the appropriate pre-calculated display coordinates, and then
calls upon the modeling and rendering routines.                  */

void draw_4poly(void){
x1=B11[q1][0];y01=B11[q1][1];z1=B11[q1][2];   /* get view coords */
x2=B11[q2][0];y2=B11[q2][1];z2=B11[q2][2];    /* get view coords */
x3=B12[q2][0];y3=B12[q2][1];z3=B12[q2][2];    /* get view coords */
x4=B12[q1][0];y4=B12[q1][1];z4=B12[q1][2];    /* get view coords */
visibility_test();if (sp>-100) return;        /* abort if hidden */
sx1=B21[q1][0];sy1=B21[q1][1];        /* retrieve display coords */
sx2=B21[q2][0];sy2=B21[q2][1];        /* retrieve display coords */
sx3=B22[q2][0];sy3=B22[q2][1];        /* retrieve display coords */
sx4=B22[q1][0];sy4=B22[q1][1];        /* retrieve display coords */
polycenter();           /* calculate area fill center of polygon */
draw_poly();                               /* draw solid polygon */
return;}

/*_______________________________________________________________


SUBROUTINE: DRAW 3-sided POLYGONAL SURFACE ON SPHERE
This subroutine retrieves the pre-calculated view coordinates
from the appropriate arrays, tests for hidden surfaces, retrieves
the appropriate pre-calculated display coordinates, and then
calls upon the modeling and rendering routines.                  */

void draw_3polynorth(void){
x1=B12[q2][0];y01=B12[q2][1];z1=B12[q2][2];   /* get view coords */
x2=B12[q1][0];y2=B12[q1][1];z2=B12[q1][2];    /* get view coords */
x3=B11[0][0];y3=B11[0][1];z3=B11[0][2];       /* get view coords */
visibility_test();if (sp>0) return;           /* abort if hidden */
sx1=B22[q2][0];sy1=B22[q2][1];        /* retrieve display coords */
sx2=B22[q1][0];sy2=B22[q1][1];        /* retrieve display coords */
sx3=B21[0][0];sy3=B21[0][1];          /* retrieve display coords */
sx4=sx1;sy4=sy1;                     /* redundant display coords */
x4=x1+.5*(x2-x1);y4=y01+.5*(y2-y01);z4=z1+.5*(z2-z1);
x=x3+.85*(x4-x3);y=y3+.85*(y4-y3);z=z3+.85*(z4-z3);
sx=d*x/z;sy=d*y/z;window();sx5=sx;sy5=sy;    /* area fill coords */
draw_poly();                               /* draw solid polygon */
return;}

void draw_3polysouth(void){
x1=B12[q1][0];y01=B12[q1][1];z1=B12[q1][2];   /* get view coords */
x2=B12[q2][0];y2=B12[q2][1];z2=B12[q2][2];    /* get view coords */
x3=B11[0][0];y3=B11[0][1];z3=B11[0][2];       /* get view coords */
visibility_test();if (sp>0) return;           /* abort if hidden */
sx1=B22[q2][0];sy1=B22[q2][1];        /* retrieve display coords */
sx2=B22[q1][0];sy2=B22[q1][1];        /* retrieve display coords */
sx3=B21[0][0];sy3=B21[0][1];          /* retrieve display coords */
sx4=sx1;sy4=sy1;                     /* redundant display coords */
x4=x1+.5*(x2-x1);y4=y01+.5*(y2-y01);z4=z1+.5*(z2-z1);
x=x3+.85*(x4-x3);y=y3+.85*(y4-y3);z=z3+.85*(z4-z3);
sx=d*x/z;sy=d*y/z;window();sx5=sx;sy5=sy;    /* area fill coords */
draw_poly();                               /* draw solid polygon */
return;}

/*_______________________________________________________________


SUBROUTINE: IDENTIFY CENTER OF POLYGON                           */

void polycenter(void){x6=x2+.5*(x1-x2);y6=y2+.5*(y01-y2);
z6=z2+.5*(z1-z2);x7=x3+.5*(x4-x3);y7=y3+.5*(y4-y3);z7=z3+.5*(z4-z3);
x=x7+.5*(x6-x7);y=y7+.5*(y6-y7);z=z7+.5*(z6-z7);sx=d*x/z;sy=d*y/z;
window();sx5=sx;sy5=sy;return;}

/*_______________________________________________________________


SUBROUTINE: CALCULATE SIN, COS FACTORS                           */

void rotation(void){
sr1=sin(r1);sr2=sin(r2);sr3=sin(r3);cr1=cos(r1);cr2=cos(r2);
cr3=cos(r3);return;}

/*_______________________________________________________________


SUBROUTINE: STANDARD 3D FORMULAS                                 */

void calc_3d(void){
x=(-1)*x;xa=cr1*x-sr1*z;za=sr1*x+cr1*z;x=cr2*xa+sr2*y;
ya=cr2*y-sr2*xa;z=cr3*za-sr3*ya;y=sr3*za+cr3*ya;x=x+mx;y=y+my;
z=z+mz;sx=d*x/z;sy=d*y/z;return;}

/*_______________________________________________________________


SUBROUTINE: HIDDEN SURFACE VISIBILITY TEST                       */

void visibility_test(void){
sp1=x1*(y2*z3-y3*z2);sp1=(-1)*sp1;sp2=x2*(y3*z1-y01*z3);
sp3=x3*(y01*z2-y2*z1);sp=sp1-sp2-sp3;return;}

/*_______________________________________________________________


SUBROUTINE: DRAW 4-SIDED SOLID POLYGON IN 3D SPACE               */

void draw_poly(void){
setlinestyle(USERBIT_LINE,0xffff,NORM_WIDTH);
setcolor(key_matte_clr);moveto(sx1,sy1);
lineto(sx2,sy2);lineto(sx3,sy3);lineto(sx4,sy4);lineto(sx1,sy1);
setfillstyle(SOLID_FILL,key_matte_clr);floodfill(sx5,sy5,key_matte_clr);
setcolor(edge_clr);moveto(sx1,sy1);
lineto(sx2,sy2);lineto(sx3,sy3);lineto(sx4,sy4);lineto(sx1,sy1);
setfillstyle(SOLID_FILL,solid_clr);floodfill(sx5,sy5,edge_clr);return;}

/*_______________________________________________________________


SUBROUTINE: MAP CARTESIAN COORDS TO PHYSICAL SCREEN COORDS       */

void window(void){
sx=sx+399;sy=sy+299;rx=screen_x/799;ry=screen_y/599;sx=sx*rx;
sy=sy*ry;return;}

/*_______________________________________________________________


SUBROUTINE: CHECK THE KEYBOARD BUFFER                            */

void keyboard(void){
if (bioskey(1)==0) return; else quit_pgm();}

/*_______________________________________________________________


SUBROUTINE: GRACEFUL EXIT FROM THE PROGRAM                       */

void quit_pgm(void){
cleardevice();restorecrtmode();exit(0);}

/*_______________________________________________________________


SUBROUTINE: VGA/EGA/CGA/MCGA compatibility module                */

void graphics_setup(void){
int graphics_adapter,graphics_mode;
detectgraph(&graphics_adapter,&graphics_mode);
if (graphics_adapter==VGA) goto VGA_mode;              /* if VGA */
if (graphics_mode==EGAHI) goto EGA_ECD_mode;   /* if EGA and ECD */
if (graphics_mode==EGALO) goto EGA_SCD_mode;   /* if EGA and SCD */
if (graphics_adapter==CGA) goto abort_message;         /* if CGA */
if (graphics_adapter==MCGA) goto abort_message;       /* if MCGA */
goto abort_message;              /* if no VGA, EGA, CGA, or MCGA */

VGA_mode:
graphics_adapter=VGA;graphics_mode=VGAHI;
initgraph(&graphics_adapter,&graphics_mode,"");
x_res=640;y_res=480;mode_flag=1;
			maxx=639;minx=0;maxy=479;miny=0;screen_x=639;screen_y=479;
			setcolor(7);moveto(0,472);
			outtext("640x480 16-color VGA mode");
			moveto(472,472);
			outtext("Press any key to quit");
			moveto(168,0);
			outtext("USING C TO GENERATE A SOLID 3D SPHERE");
			return;

EGA_ECD_mode:
graphics_adapter=EGA;graphics_mode=EGAHI;
initgraph(&graphics_adapter,&graphics_mode,"");
x_res=640;y_res=350;mode_flag=2;
			maxx=639;minx=0;maxy=349;miny=0;screen_x=639;screen_y=349;
			setcolor(7);moveto(0,342);
			outtext("640x350 16-color EGA mode");
			moveto(472,342);
			outtext("Press any key to quit");
			moveto(168,0);
			outtext("USING C TO GENERATE A SOLID 3D SPHERE");
			return;

EGA_SCD_mode:
graphics_adapter=EGA;graphics_mode=EGALO;
initgraph(&graphics_adapter,&graphics_mode,"");
x_res=640;y_res=200;mode_flag=3;
			maxx=639;minx=0;maxy=199;miny=0;screen_x=639;screen_y=199;
			setcolor(7);moveto(0,192);
			outtext("640x200 16-color EGA mode");
			moveto(472,192);
			outtext("Press any key to quit");
			moveto(168,0);
			outtext("USING C TO GENERATE A SOLID 3D SPHERE");
			return;

abort_message:
printf("\n\nUnable to proceed.\n");
printf("Requires VGA, EGA, CGA, or MCGA adapter\n");
printf("   with appropriate monitor.\n");
printf("Please refer to the book.\n\n");
exit(0);
}

/*_______________________________________________________________


SUBROUTINE: MAP 640x480 TEMPLATE TO 2D SCREEN                    */

void coords(void){sx=sx*(x_res/640);sy=sy*(y_res/480);return;}

/*_______________________________________________________________


SUBROUTINE: Copyright Notice                                     */

int copyright[][3]={0x7c00,0x0000,0x0000,0x8231,
0x819c,0x645e,0xba4a,0x4252,0x96d0,0xa231,0x8252,0x955e,0xba4a,
0x43d2,0xf442,0x8231,0x825c,0x945e,0x7c00,0x0000,0x0000};

void notice(int x, int y){
int a,b,c; int t1=0;
for (t1=0;t1<=6;t1++){a=copyright[t1][0];b=copyright[t1][1];
c=copyright[t1][2];
setlinestyle(USERBIT_LINE,a,NORM_WIDTH);
moveto(x,y);lineto(x+15,y);
setlinestyle(USERBIT_LINE,b,NORM_WIDTH);
moveto(x+16,y);lineto(x+31,y);
setlinestyle(USERBIT_LINE,c,NORM_WIDTH);
moveto(x+32,y);lineto(x+47,y);y=y+1;};
setlinestyle(USERBIT_LINE,0xFFFF,NORM_WIDTH);
return;}

/*_______________________________________________________________


End of source code                                               */


